@@ -18,7 +18,6 @@ from abc import ABC, abstractmethod
 from .vector import HybridVectorStore
 from .llm import BaseLLMClient
 # Lazy import of ContextEngineer to avoid circular dependency
-# from .context_engineer import ContextEngineer
 
 logger = logging.getLogger(__name__)
 
@@ -215,7 +214,7 @@ class DynamicTaskGraph:
             },
             'complex_analytical': {
                 'description': 'Complex analytical query requiring multi-step processing',
-                'tasks': ['fetch_data', 'budget_allocation', 'parallel_retrieve', 'compress_context', 'analyze_data', 'generate_insights'],
+                'tasks': ['fetch_data', 'budget_allocation', 'retrieve_docs', 'retrieve_metadata', 'assess_quality', 'compress_context', 'analyze_data', 'generate_insights'],
                 'parallel_groups': [['retrieve_docs', 'retrieve_metadata', 'assess_quality']],
                 'conditional_tasks': {
                     'compress_context': 'budget_allocation.compression_needed',
@@ -313,12 +312,26 @@ class DynamicTaskGraph:
         tasks = []
         complexity = query_analysis['estimated_processing_complexity']
         
+        # Process sequential tasks
         for task_name in base_template['tasks']:
             task_config = self._get_adaptive_task_config(task_name, query_analysis, complexity)
             task = self._create_task_instance(task_name, task_config, context)
             if task:
+                # Add dependencies for tasks following parallel groups
+                if task_name in base_template.get('tasks', []) and 'parallel_groups' in base_template:
+                    parallel_tasks = [t for group in base_template['parallel_groups'] for t in group]
+                    if task_name not in parallel_tasks:
+                        # Find the index of the task
+                        task_index = base_template['tasks'].index(task_name)
+                        # Add dependencies from the last parallel group before this task
+                        for group in base_template['parallel_groups']:
+                            if base_template['tasks'].index(group[0]) < task_index:
+                                task.metadata.dependencies.update(
+                                    {f"{t}_{datetime.utcnow().timestamp()}" for t in group}
+                                )
                 tasks.append(task)
         
+        # Process conditional tasks
         for task_name, condition in base_template.get('conditional_tasks', {}).items():
             if self._evaluate_task_condition(condition, query_analysis, context):
                 task_config = self._get_adaptive_task_config(task_name, query_analysis, complexity)
@@ -350,6 +363,18 @@ class DynamicTaskGraph:
                 'timeout_seconds': base_config['timeout_seconds'] * 1.5,
                 'description': 'Retrieve and filter contextual documents'
             },
+            'retrieve_docs': {
+                'timeout_seconds': base_config['timeout_seconds'] * 1.2,
+                'description': 'Retrieve document content for analysis'
+            },
+            'retrieve_metadata': {
+                'timeout_seconds': base_config['timeout_seconds'] * 1.0,
+                'description': 'Retrieve metadata for documents'
+            },
+            'assess_quality': {
+                'timeout_seconds': base_config['timeout_seconds'] * 1.3,
+                'description': 'Assess quality of retrieved documents'
+            },
             'compress_context': {
                 'timeout_seconds': base_config['timeout_seconds'] * 2.0,
                 'description': 'Compress context using RL optimization'
@@ -377,6 +402,9 @@ class DynamicTaskGraph:
             'fetch_data': FetchDataTask,
             'budget_allocation': BudgetAllocationTask,
             'retrieve_context': lambda metadata: RetrieveContextTask(metadata, self.vector_store),
+            'retrieve_docs': lambda metadata: RetrieveDocsTask(metadata, self.vector_store),
+            'retrieve_metadata': lambda metadata: RetrieveMetadataTask(metadata, self.vector_store),
+            'assess_quality': AssessQualityTask,
             'compress_context': CompressContextTask,
             'generate_response': lambda metadata: GenerateResponseTask(metadata, self.llm_optimizer, self.context_engineer),
             'analyze_data': AnalyzeDataTask,
@@ -587,17 +615,103 @@ class RetrieveContextTask(Task):
                 error=e
             )
 
+class RetrieveDocsTask(Task):
+    def __init__(self, metadata: TaskMetadata, vector_store: HybridVectorStore):
+        super().__init__(metadata)
+        self.vector_store = vector_store
+
+    async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Retrieving documents for query: {context.query}")
+        try:
+            search_results = await self.vector_store.search(query=context.query, k=10)
+            documents = [
+                {
+                    'content': result.content,
+                    'source_id': result.source_id,
+                    'score': result.fusion_score
+                } for result in search_results
+            ]
+            result = {
+                'documents': documents,
+                'total_retrieved': len(documents)
+            }
+            return TaskResult(
+                task_id=self.metadata.task_id,
+                status=TaskStatus.COMPLETED,
+                result=result,
+                context_updates={'retrieved_docs': result}
+            )
+        except Exception as e:
+            logger.error(f"Document retrieval failed: {str(e)}")
+            return TaskResult(
+                task_id=self.metadata.task_id,
+                status=TaskStatus.FAILED,
+                error=e
+            )
+
+class RetrieveMetadataTask(Task):
+    def __init__(self, metadata: TaskMetadata, vector_store: HybridVectorStore):
+        super().__init__(metadata)
+        self.vector_store = vector_store
+
+    async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Retrieving metadata for query: {context.query}")
+        try:
+            search_results = await self.vector_store.search(query=context.query, k=10)
+            metadata = [
+                {
+                    'source_id': result.source_id,
+                    'metadata': result.metadata
+                } for result in search_results
+            ]
+            result = {
+                'metadata': metadata,
+                'total_retrieved': len(metadata)
+            }
+            return TaskResult(
+                task_id=self.metadata.task_id,
+                status=TaskStatus.COMPLETED,
+                result=result,
+                context_updates={'retrieved_metadata': result}
+            )
+        except Exception as e:
+            logger.error(f"Metadata retrieval failed: {str(e)}")
+            return TaskResult(
+                task_id=self.metadata.task_id,
+                status=TaskStatus.FAILED,
+                error=e
+            )
+
+class AssessQualityTask(Task):
+    async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Assessing quality for query: {context.query}")
+        await asyncio.sleep(0.2)
+        retrieved_docs = context.processed_data.get('retrieved_docs', {}).get('documents', [])
+        quality_scores = [
+            {'source_id': doc['source_id'], 'quality_score': doc.get('score', 0.8) * 0.9}
+            for doc in retrieved_docs
+        ]
+        result = {
+            'quality_scores': quality_scores,
+            'average_quality': sum(score['quality_score'] for score in quality_scores) / len(quality_scores) if quality_scores else 0.8
+        }
+        return TaskResult(
+            task_id=self.metadata.task_id,
+            status=TaskStatus.COMPLETED,
+            result=result,
+            context_updates={'quality_assessment': result}
+        )
+
 class CompressContextTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
         logger.info(f"Compressing context for query: {context.query}")
-        retrieved_context = context.processed_data.get('retrieved_context', {})
+        retrieved_docs = context.processed_data.get('retrieved_docs', {}).get('documents', [])
         budget_allocation = context.budget_allocation
         await asyncio.sleep(0.4)
-        documents = retrieved_context.get('filtered_documents', [])
-        compressed_content = f"Compressed summary of {len(documents)} documents for query: {context.query}"
+        compressed_content = f"Compressed summary of {len(retrieved_docs)} documents for query: {context.query}"
         compression_result = {
             'compressed_content': compressed_content,
-            'original_token_count': sum(len(doc.get('content', '').split()) for doc in documents),
+            'original_token_count': sum(len(doc.get('content', '').split()) for doc in retrieved_docs),
             'compressed_token_count': len(compressed_content.split()),
             'compression_ratio': 0.3,
             'quality_score': 0.85
@@ -618,10 +732,8 @@ class GenerateResponseTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
         logger.info(f"Generating response for query: {context.query}")
         try:
-            retrieved_context = context.processed_data.get('retrieved_context', {})
+            retrieved_docs = context.processed_data.get('retrieved_docs', {}).get('documents', [])
             budget = context.budget_allocation
-            documents = retrieved_context.get('filtered_documents', [])
-            # Create a QueryComplexity object for logging and passing to context_engineer
             complexity = QueryComplexity(
                 overall_score=0.5,
                 semantic_complexity=0.4,
@@ -629,11 +741,10 @@ class GenerateResponseTask(Task):
                 temporal_complexity=0.2,
                 domain_complexity=0.3
             )
-            # Log complexity details safely
             logger.debug(f"Query complexity for {context.query}: {asdict(complexity)}")
             prompt = await self.context_engineer.build_prompt(
                 query=context.query,
-                raw_docs=documents,
+                raw_docs=retrieved_docs,
                 budget=budget,
                 semantic_state=context.semantic_state,
                 complexity=complexity
@@ -725,7 +836,7 @@ class DetectAnomaliesTask(Task):
 class DAGEngine:
     def __init__(self, max_parallel_tasks: int = 10, vector_store=None, llm_optimizer=None, context_engineer=None, templates_dir: Optional[str] = None):
         if context_engineer is None:
-            from .context_engineer import ContextEngineer, PromptConstructionConfig  # Lazy import
+            from .context_engineer import ContextEngineer, PromptConstructionConfig
             context_engineer = ContextEngineer(templates_dir=templates_dir)
         self.task_graph_generator = DynamicTaskGraph(vector_store, llm_optimizer, context_engineer)
         self.context_manager = SemanticContextManager()
